@@ -26,6 +26,8 @@
 #include <ctype.h>
 #include "e-vcard.h"
 
+#define d(x)
+
 #define CRLF "\r\n"
 
 typedef enum {
@@ -269,7 +271,8 @@ read_attribute_value (EVCardAttribute *attr, char **p, gboolean quoted_printable
 			}
 			lp = g_utf8_next_char(lp);
 		}
-		else if (*lp == ';') {
+		else if ((*lp == ';') ||
+			 (*lp == ',' && !strcmp (attr->name, "CATEGORIES"))) {
 			e_vcard_attribute_add_value (attr, str->str);
 			g_string_assign (str, "");
 			lp = g_utf8_next_char(lp);
@@ -522,16 +525,13 @@ parse (EVCard *evc, const char *str)
 		*end = '\0';
 	}
 	
-#if DEBUG_FOLDING
-	printf ("BEFORE FOLDING:\n");
-	printf (str);
-#endif
 	buf = fold_lines (buf);
 
-#if DEBUG_FOLDING
-	printf ("\n\nAFTER FOLDING:\n");
-	printf (buf);
-#endif
+	d(printf ("BEFORE FOLDING:\n"));
+	d(printf (str));
+	d(printf ("\n\nAFTER FOLDING:\n"));
+	d(printf (buf));
+
 	p = buf;
 
 	attr = read_attribute (&p);
@@ -672,7 +672,7 @@ e_vcard_to_string_vcard_30 (EVCard *evc)
 
 	GString *str = g_string_new ("");
 
-	str = g_string_append (str, "BEGIN:vCard" CRLF);
+	str = g_string_append (str, "BEGIN:VCARD" CRLF);
 
 	/* we hardcode the version (since we're outputting to a
 	   specific version) and ignore any version attributes the
@@ -743,8 +743,15 @@ e_vcard_to_string_vcard_30 (EVCard *evc)
 			escaped_value = e_vcard_escape_string (value);
 
 			attr_str = g_string_append (attr_str, escaped_value);
-			if (v->next)
-				attr_str = g_string_append_c (attr_str, ';');
+			if (v->next) {
+				/* XXX toshok - i hate you, rfc 2426.
+				   why doesn't CATEGORIES use a ; like
+				   a normal list attribute? */
+				if (!strcmp (attr->name, "CATEGORIES"))
+					attr_str = g_string_append_c (attr_str, ',');
+				else
+					attr_str = g_string_append_c (attr_str, ';');
+			}
 
 			g_free (escaped_value);
 		}
@@ -769,7 +776,7 @@ e_vcard_to_string_vcard_30 (EVCard *evc)
 		g_string_free (attr_str, TRUE);
 	}
 
-	str = g_string_append (str, "END:vCard");
+	str = g_string_append (str, "END:VCARD");
 
 	return g_string_free (str, FALSE);
 }
@@ -950,8 +957,8 @@ e_vcard_attribute_add_value_decoded (EVCardAttribute *attr, const char *value, i
 		/* make sure the decoded list is up to date */
 		e_vcard_attribute_get_values_decoded (attr);
 
-		printf ("base64 encoded value: %s\n", b64_data);
-		printf ("original length: %d\n", len);
+		d(printf ("base64 encoded value: %s\n", b64_data));
+		d(printf ("original length: %d\n", len));
 
 		attr->values = g_list_append (attr->values, b64_data);
 		attr->decoded_values = g_list_append (attr->decoded_values, decoded);
@@ -990,9 +997,11 @@ e_vcard_attribute_remove_values (EVCardAttribute *attr)
 {
 	g_list_foreach (attr->values, (GFunc)g_free, NULL);
 	g_list_free (attr->values);
+	attr->values = NULL;
+
 	g_list_foreach (attr->decoded_values, (GFunc)free_gstring, NULL);
 	g_list_free (attr->decoded_values);
-	attr->values = NULL;
+	attr->decoded_values = NULL;
 }
 
 void
@@ -1001,6 +1010,10 @@ e_vcard_attribute_remove_params (EVCardAttribute *attr)
 	g_list_foreach (attr->params, (GFunc)e_vcard_attribute_param_free, NULL);
 	g_list_free (attr->params);
 	attr->params = NULL;
+
+	/* also remove the cached encoding on this attribute */
+	attr->encoding_set = FALSE;
+	attr->encoding = EVC_ENCODING_RAW;
 }
 
 EVCardAttributeParam*
@@ -1176,6 +1189,65 @@ e_vcard_attribute_get_values_decoded (EVCardAttribute *attr)
 	return attr->decoded_values;
 }
 
+gboolean
+e_vcard_attribute_is_single_valued (EVCardAttribute *attr)
+{
+	if (attr->values == NULL
+	    || attr->values->next != NULL)
+		return FALSE;
+
+	return TRUE;
+}
+
+char*
+e_vcard_attribute_get_value (EVCardAttribute *attr)
+{
+	GList *values = e_vcard_attribute_get_values (attr);
+
+	if (!e_vcard_attribute_is_single_valued (attr))
+		g_warning ("e_vcard_attribute_get_value called on multivalued attribute");
+
+	return values ? g_strdup ((char*)values->data) : NULL;
+}
+
+GString*
+e_vcard_attribute_get_value_decoded (EVCardAttribute *attr)
+{
+	GList *values = e_vcard_attribute_get_values_decoded (attr);
+	GString *str = NULL;
+
+	if (!e_vcard_attribute_is_single_valued (attr))
+		g_warning ("e_vcard_attribute_get_value_decoded called on multivalued attribute");
+
+	if (values)
+		str = values->data;
+
+	return str ? g_string_new_len (str->str, str->len) : NULL;
+}
+
+gboolean
+e_vcard_attribute_has_type (EVCardAttribute *attr, const char *typestr)
+{
+	GList *params = e_vcard_attribute_get_params (attr);
+	GList *p;
+
+	for (p = params; p; p = p->next) {
+		EVCardAttributeParam *param = p->data;
+
+		if (!strcasecmp (e_vcard_attribute_param_get_name (param), EVC_TYPE)) {
+			GList *values = e_vcard_attribute_param_get_values (param);
+			GList *v;
+
+			for (v = values; v; v = v->next) {
+				if (!strcasecmp ((char*)v->data, typestr))
+					return TRUE;
+			}
+		}
+	}
+
+	return FALSE;
+}
+
 GList*
 e_vcard_attribute_get_params (EVCardAttribute *attr)
 {
@@ -1229,12 +1301,10 @@ _evc_base64_encode_close(unsigned char *in, size_t inlen, gboolean break_lines,
 	c1 = ((unsigned char *)save)[1];
 	c2 = ((unsigned char *)save)[2];
 	
-#if 0
 	d(printf("mode = %d\nc1 = %c\nc2 = %c\n",
 		 (int)((char *)save)[0],
 		 (int)((char *)save)[1],
 		 (int)((char *)save)[2]));
-#endif
 
 	switch (((char *)save)[0]) {
 	case 2:
@@ -1275,9 +1345,7 @@ _evc_base64_encode_step(unsigned char *in, size_t len, gboolean break_lines, uns
 	inptr = in;
 	outptr = out;
 
-#if 0
 	d(printf("we have %d chars, and %d saved chars\n", len, ((char *)save)[0]));
-#endif
 
 	if (len + ((char *)save)[0] > 2) {
 		unsigned char *inend = in+len-2;
@@ -1315,11 +1383,9 @@ _evc_base64_encode_step(unsigned char *in, size_t len, gboolean break_lines, uns
 		*state = already;
 	}
 
-#if 0
 	d(printf("state = %d, len = %d\n",
 		 (int)((char *)save)[0],
 		 len));
-#endif
 
 	if (len>0) {
 		register char *saveout;
@@ -1335,12 +1401,10 @@ _evc_base64_encode_step(unsigned char *in, size_t len, gboolean break_lines, uns
 		((char *)save)[0]+=len;
 	}
 
-#if 0
 	d(printf("mode = %d\nc1 = %c\nc2 = %c\n",
 		 (int)((char *)save)[0],
 		 (int)((char *)save)[1],
 		 (int)((char *)save)[2]));
-#endif
 
 	return outptr-out;
 }
