@@ -22,6 +22,7 @@
 #define E_CAL_H
 
 #include <glib-object.h>
+#include <libedataserver/e-source-list.h>
 #include <libedataserver/e-source.h>
 #include <libecal/e-cal-recur.h>
 #include <libecal/e-cal-util.h>
@@ -45,6 +46,13 @@ typedef struct _ECal ECal;
 typedef struct _ECalClass ECalClass;
 typedef struct _ECalPrivate ECalPrivate;
 
+typedef enum {
+	E_CAL_SOURCE_TYPE_EVENT,
+	E_CAL_SOURCE_TYPE_TODO,
+	E_CAL_SOURCE_TYPE_JOURNAL,
+	E_CAL_SOURCE_TYPE_LAST
+} ECalSourceType;
+
 /* Set mode status for the e_cal_set_mode function */
 typedef enum {
 	E_CAL_SET_MODE_SUCCESS,
@@ -55,6 +63,7 @@ typedef enum {
 /* Whether the ecal is not loaded, is being loaded, or is already loaded */
 typedef enum {
 	E_CAL_LOAD_NOT_LOADED,
+	E_CAL_LOAD_AUTHENTICATING,
 	E_CAL_LOAD_LOADING,
 	E_CAL_LOAD_LOADED
 } ECalLoadState;
@@ -74,19 +83,16 @@ struct _ECalClass {
 	void (* cal_opened) (ECal *ecal, ECalendarStatus status);
 	void (* cal_set_mode) (ECal *ecal, ECalSetModeStatus status, CalMode mode);	
 
-	void (* backend_error) (ECal *ecal, const char *message);
-
 	void (* categories_changed) (ECal *ecal, GPtrArray *categories);
 
-	void (* forget_password) (ECal *ecal, const char *key);
-
+	void (* backend_error) (ECal *ecal, const char *message);
 	void (* backend_died) (ECal *ecal);
 };
 
-typedef gchar * (* ECalAuthFunc) (ECal *ecal,
-				  const gchar *prompt,
-				  const gchar *key,
-				  gpointer user_data);
+typedef char * (* ECalAuthFunc) (ECal *ecal,
+				 const char *prompt,
+				 const char *key,
+				 gpointer user_data);
 
 GType e_cal_get_type (void);
 
@@ -94,19 +100,22 @@ GType e_cal_open_status_enum_get_type (void);
 GType e_cal_set_mode_status_enum_get_type (void);
 GType cal_mode_enum_get_type (void);
 
-ECal *e_cal_new (ESource *source, CalObjType type);
-ECal *e_cal_new_from_uri (const gchar *uri, CalObjType type);
+ECal *e_cal_new (ESource *source, ECalSourceType type);
+ECal *e_cal_new_from_uri (const gchar *uri, ECalSourceType type);
+ECal *e_cal_new_system_calendar (void);
+ECal *e_cal_new_system_tasks (void);
 
 void e_cal_set_auth_func (ECal *ecal, ECalAuthFunc func, gpointer data);
 
 gboolean e_cal_open (ECal *ecal, gboolean only_if_exists, GError **error);
 void e_cal_open_async (ECal *ecal, gboolean only_if_exists);
-gboolean e_cal_remove_calendar (ECal *ecal, GError **error);
+gboolean e_cal_remove (ECal *ecal, GError **error);
 
 GList *e_cal_uri_list (ECal *ecal, CalMode mode);
 
 ECalLoadState e_cal_get_load_state (ECal *ecal);
 
+ESource *e_cal_get_source (ECal *ecal);
 const char *e_cal_get_uri (ECal *ecal);
 
 gboolean e_cal_is_read_only (ECal *ecal, gboolean *read_only, GError **error);
@@ -140,8 +149,7 @@ void e_cal_free_object_list (GList *objects);
 gboolean e_cal_get_free_busy (ECal *ecal, GList *users, time_t start, time_t end, 
 			      GList **freebusy, GError **error);
 
-void e_cal_generate_instances (ECal *ecal, CalObjType type,
-			       time_t start, time_t end,
+void e_cal_generate_instances (ECal *ecal, time_t start, time_t end,
 			       ECalRecurInstanceFn cb, gpointer cb_data);
 
 GSList *e_cal_get_alarms_in_range (ECal *ecal, time_t start, time_t end);
@@ -160,7 +168,7 @@ gboolean e_cal_remove_object_with_mod (ECal *ecal, const char *uid, const char *
 gboolean e_cal_discard_alarm (ECal *ecal, ECalComponent *comp, const char *auid, GError **error);
 
 gboolean e_cal_receive_objects (ECal *ecal, icalcomponent *icalcomp, GError **error);
-gboolean e_cal_send_objects (ECal *ecal, icalcomponent *icalcomp, GError **error);
+gboolean e_cal_send_objects (ECal *ecal, icalcomponent *icalcomp, GList **users, icalcomponent **modified_icalcomp, GError **error);
 
 gboolean e_cal_get_timezone (ECal *ecal, const char *tzid, icaltimezone **zone, GError **error);
 gboolean e_cal_add_timezone (ECal *ecal, icaltimezone *izone, GError **error);
@@ -181,6 +189,12 @@ char* e_cal_get_component_as_string (ECal *ecal, icalcomponent *icalcomp);
 
 const char * e_cal_get_error_message (ECalendarStatus status);
 
+/* Calendar/Tasks Discovery */
+gboolean    e_cal_open_default (ECal **ecal, ECalSourceType type, ECalAuthFunc func, gpointer data, GError **error);
+gboolean    e_cal_set_default (ECal  *ecal, GError **error);
+gboolean    e_cal_set_default_source (ESource *source, ECalSourceType type, GError **error);
+gboolean    e_cal_get_sources (ESourceList **sources, ECalSourceType type, GError **error);
+
 G_END_DECLS
 
 #endif
